package com.rubin.mybatis.builder;

import com.mchange.v2.c3p0.ComboPooledDataSource;
import com.rubin.mybatis.io.Resource;
import com.rubin.mybatis.pojo.Configuration;
import lombok.AllArgsConstructor;
import lombok.Data;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import java.beans.PropertyVetoException;
import java.io.File;
import java.util.Properties;

/**
 * 配置读取器
 * Created by rubin on 2021/3/6.
 */
@Data
@AllArgsConstructor
public class ConfigurationBuilder extends AbstractBuilder {

    private Configuration configuration;

    /**
     * 根据配置文件名称读取数据源配置和映射mapper配置
     *
     * @param configLocation 配置文件名称 配置文件需放在classpath下面
     * @throws DocumentException
     * @throws PropertyVetoException
     */
    public void parse(String configLocation) throws DocumentException, PropertyVetoException, ClassNotFoundException {
        File configFile = getConfigFile(configLocation);
        Document document = new SAXReader().read(configFile);
        Element root = document.getRootElement();
        readDatasource(root);
        readMapperStatements(configLocation);
    }

    /**
     * 读取mapper信息
     *
     * @param configLocation
     */
    private void readMapperStatements(String configLocation) throws ClassNotFoundException, DocumentException {
        new MapperStatementBuilder(configuration).parse(configLocation);
    }

    /**
     * 读取数据源信息
     *
     * @param root
     * @throws PropertyVetoException
     */
    private void readDatasource(Element root) throws PropertyVetoException {
        Element dataSourceElement = root.element("dataSource");
        Properties properties = readElementAsProperties(dataSourceElement);

        ComboPooledDataSource comboPooledDataSource = new ComboPooledDataSource();
        comboPooledDataSource.setDriverClass(properties.getProperty("driverClass"));
        comboPooledDataSource.setJdbcUrl(properties.getProperty("jdbcUrl"));
        comboPooledDataSource.setUser(properties.getProperty("user"));
        comboPooledDataSource.setPassword(properties.getProperty("password"));

        configuration.setDataSource(comboPooledDataSource);
    }

    /**
     * 获取配置文件实体
     *
     * @param configLocation
     * @return
     */
    private File getConfigFile(String configLocation) {
        return Resource.getResource(configLocation);
    }

}
