package com.rubin.mybatis.sqlsession;

import com.rubin.mybatis.builder.MapperStatementBuilder;
import com.rubin.mybatis.connection.ConnectionFactory;
import com.rubin.mybatis.pojo.Configuration;
import lombok.Data;
import org.dom4j.DocumentException;

import java.sql.SQLException;

/**
 * SqlSession工厂的默认是实现类
 * Created by rubin on 2021/3/6.
 */
@Data
public class DefaultSqlSessionFactory implements SqlSessionFactory {

    private Configuration configuration;

    private ConnectionFactory connectionFactory;

    private String mapperBasePackage;

    public DefaultSqlSessionFactory() {
    }

    public DefaultSqlSessionFactory(Configuration configuration, ConnectionFactory connectionFactory) {
        this.configuration = configuration;
        this.connectionFactory = connectionFactory;
    }

    public void setMapperBasePackage(String mapperBasePackage) {
        this.mapperBasePackage = mapperBasePackage;
    }

    /**
     * 生产SqlSession
     *
     * @return
     */
    @Override
    public SqlSession openSession() throws SQLException {
        SqlSession sqlSession = new DefaultSqlSession(configuration, connectionFactory);
        return sqlSession;
    }

    /**
     * 初始化方法 便于容器调用
     *
     * @throws ClassNotFoundException
     * @throws DocumentException
     */
    public void init() throws ClassNotFoundException, DocumentException {
        new MapperStatementBuilder(configuration).parseList(mapperBasePackage);
    }

}
