package com.rubin.mybatis.sqlsession;

import com.rubin.mybatis.pojo.MapperStatement;

import java.sql.SQLException;
import java.util.List;

/**
 * SqlSession接口
 * Created by rubin on 2021/3/6.
 */
public interface SqlSession {

    /**
     * 查询列表
     *
     * @param mapperStatement
     * @param arg
     * @return
     */
    <E> List<E> selectList(MapperStatement mapperStatement, Object arg) throws Exception;

    /**
     * 查询单条记录
     *
     * @param mapperStatement
     * @param arg
     * @return
     */
    <T> T selectOne(MapperStatement mapperStatement, Object arg) throws Exception;

    /**
     * 插入单条记录
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws Exception
     */
    int insert(MapperStatement mapperStatement, Object arg) throws Exception;

    /**
     * 更新单条记录
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws Exception
     */
    int update(MapperStatement mapperStatement, Object arg) throws Exception;

    /**
     * 删除单挑记录
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws Exception
     */
    int delete(MapperStatement mapperStatement, Object arg) throws Exception;

    /**
     * 获取mapper接口的代理类 让用户无感知的调用本地接口
     *
     * @param mapperClass
     * @param <T>
     * @return
     */
    <T> T getMapper(Class<T> mapperClass);

    /**
     * 关闭连接
     *
     * @throws SQLException
     */
    void close() throws SQLException;

}
