package com.rubin.mybatis.sqlsession;

import com.rubin.mybatis.builder.ConfigurationBuilder;
import com.rubin.mybatis.builder.ConnectionFactoryBuilder;
import com.rubin.mybatis.pojo.Configuration;
import lombok.Data;
import org.dom4j.DocumentException;

import java.beans.PropertyVetoException;

/**
 * SqlSession工厂构造器
 * Created by rubin on 2021/3/6.
 */
@Data
public class SqlSessionFactoryBuilder {

    private Configuration configuration;

    public SqlSessionFactoryBuilder() {
        this.configuration = new Configuration();
    }

    /**
     * 构造SqlSession工厂
     * 包括：
     * 1、 读取配置信息
     * 2、 创建SqlSessionFactory默认实体
     *
     * @param configLocation
     * @return
     * @throws DocumentException
     * @throws PropertyVetoException
     * @throws ClassNotFoundException
     */
    public SqlSessionFactory build(String configLocation) throws DocumentException, PropertyVetoException, ClassNotFoundException {
        ConfigurationBuilder configurationBuilder = new ConfigurationBuilder(configuration);
        configurationBuilder.parse(configLocation);
        SqlSessionFactory sqlSessionFactory = new DefaultSqlSessionFactory(configuration, new ConnectionFactoryBuilder().build(configuration.getDataSource()));
        return sqlSessionFactory;
    }

}
