package com.rubin.springboot.server;

import com.rubin.spring.context.utils.StringUtil;
import org.apache.catalina.LifecycleException;
import org.apache.catalina.startup.Tomcat;

import javax.servlet.ServletException;

/**
 * Created by RubinChu on 2021/4/28 20:20
 */
public class TomcatWebServer implements WebServer {

    private Tomcat tomcat;

    private Integer port = 8080;

    private String configLocation = Thread.currentThread().getContextClassLoader().getResource("").getPath();

    public TomcatWebServer() {
        this(null);
    }

    public TomcatWebServer(Integer port) {
        this(port, null);
    }

    public TomcatWebServer(Integer port, String configLocation) {
        if (port != null) {
            this.port = port;
        }
        if (StringUtil.isNotBlank(configLocation)) {
            this.configLocation = configLocation;
        }
        initTomcat();
    }

    /**
     * 初始化tomcat
     */
    private void initTomcat() {
        tomcat = new Tomcat();
        tomcat.setPort(port);

        // 这一步很关键 如果不加这个操作  tomcat不会自动加载配置文件
        try {
            tomcat.addWebapp("/", configLocation);
        } catch (ServletException e) {
            e.printStackTrace();
        }
    }

    /**
     * 启动Web服务器
     */
    @Override
    public void start() {
        try {
            tomcat.start();
        } catch (LifecycleException e) {
            e.printStackTrace();
        }
    }

    /**
     * 保持监听状态
     */
    @Override
    public void await() {
        tomcat.getServer().await();
    }

    /**
     * 停止
     */
    @Override
    public void stop() {
        try {
            tomcat.stop();
        } catch (LifecycleException e) {
            e.printStackTrace();
        }
    }

}
