package com.rubin.client.service;

import com.rubin.client.pojo.AccountInfo;

import java.lang.reflect.UndeclaredThrowableException;
import java.math.BigDecimal;

/**
 * 账户信息处理类接口
 */
public interface IAccountService {

    /**
     * 通过卡号获取账户详情
     *
     * @param cardNo
     * @return
     */
    AccountInfo getByCardNo(String cardNo);

    /**
     * 没有事务控制 并有异常抛出
     *
     * @param fromCardNo
     * @param toCardNo
     * @param money
     * @throws UndeclaredThrowableException
     */
    void transferWithoutTransaction(String fromCardNo, String toCardNo, BigDecimal money) throws UndeclaredThrowableException;

    /**
     * 有事务控制 并有异常抛出
     *
     * @param fromCardNo
     * @param toCardNo
     * @param money
     * @throws UndeclaredThrowableException
     */
    void transferWithTransactionRollback(String fromCardNo, String toCardNo, BigDecimal money) throws UndeclaredThrowableException;

    /**
     * 有事务控制 正常执行
     *
     * @param fromCardNo
     * @param toCardNo
     * @param money
     */
    void transferWithTransactionSuccess(String fromCardNo, String toCardNo, BigDecimal money);

}
