package com.rubin.client.service.impl;

import com.rubin.client.dao.IAccountInfoDao;
import com.rubin.client.pojo.AccountInfo;
import com.rubin.client.service.IAccountService;
import com.rubin.spring.context.annotation.Autowired;
import com.rubin.spring.context.annotation.Service;
import com.rubin.spring.ex.annotation.Transactional;

import java.lang.reflect.UndeclaredThrowableException;
import java.math.BigDecimal;

/**
 * jdk动态代理测试类
 * 主要测试方法级事务的控制
 */
@Service(value = "accountService")
public class AccountServiceImpl implements IAccountService {

    @Autowired
    private IAccountInfoDao iAccountInfoDao;

    /**
     * 通过卡号获取账户详情
     *
     * @param cardNo
     * @return
     */
    @Override
    public AccountInfo getByCardNo(String cardNo) {
        AccountInfo accountInfo = new AccountInfo();
        accountInfo.setCardNo(cardNo);
        return iAccountInfoDao.selectByCardNo(accountInfo);
    }

    /**
     * 没有事务控制 并有异常抛出
     *
     * @param fromCardNo
     * @param toCardNo
     * @param money
     * @throws UndeclaredThrowableException
     */
    @Override
    public void transferWithoutTransaction(String fromCardNo, String toCardNo, BigDecimal money) throws UndeclaredThrowableException {
        AccountInfo accountInfo = new AccountInfo();
        accountInfo.setCardNo(fromCardNo);
        AccountInfo from = iAccountInfoDao.selectByCardNo(accountInfo);

        accountInfo.setCardNo(toCardNo);
        AccountInfo to = iAccountInfoDao.selectByCardNo(accountInfo);

        from.setMoney(from.getMoney().subtract(money));
        to.setMoney(to.getMoney().add(money));

        iAccountInfoDao.updateByCardNo(from);
        int i = 1 / 0;
        iAccountInfoDao.updateByCardNo(to);
    }

    /**
     * 有事务控制 并有异常抛出
     *
     * @param fromCardNo
     * @param toCardNo
     * @param money
     * @throws UndeclaredThrowableException
     */
    @Transactional
    @Override
    public void transferWithTransactionRollback(String fromCardNo, String toCardNo, BigDecimal money) throws UndeclaredThrowableException {
        AccountInfo accountInfo = new AccountInfo();
        accountInfo.setCardNo(fromCardNo);
        AccountInfo from = iAccountInfoDao.selectByCardNo(accountInfo);

        accountInfo.setCardNo(toCardNo);
        AccountInfo to = iAccountInfoDao.selectByCardNo(accountInfo);

        from.setMoney(from.getMoney().subtract(money));
        to.setMoney(to.getMoney().add(money));

        iAccountInfoDao.updateByCardNo(from);
        int i = 1 / 0;
        iAccountInfoDao.updateByCardNo(to);
    }

    /**
     * 有事务控制 正常执行
     *
     * @param fromCardNo
     * @param toCardNo
     * @param money
     */
    @Transactional
    @Override
    public void transferWithTransactionSuccess(String fromCardNo, String toCardNo, BigDecimal money) {
        AccountInfo accountInfo = new AccountInfo();
        accountInfo.setCardNo(fromCardNo);
        AccountInfo from = iAccountInfoDao.selectByCardNo(accountInfo);

        accountInfo.setCardNo(toCardNo);
        AccountInfo to = iAccountInfoDao.selectByCardNo(accountInfo);

        from.setMoney(from.getMoney().subtract(money));
        to.setMoney(to.getMoney().add(money));

        iAccountInfoDao.updateByCardNo(from);
        iAccountInfoDao.updateByCardNo(to);
    }
}
