package com.rubin.spring.mybatis.connection;

import com.rubin.mybatis.connection.ConnectionFactory;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;

/**
 * Connection工厂
 * 保证了同一个线程使用同一个连接
 */
public class ThreadConnectionFactory implements ConnectionFactory {

    private ThreadLocal<Connection> connectionThreadLocal = new ThreadLocal<>();

    public DataSource dataSource;

    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    /**
     * 生产Collection，保证了同一个线程使用同一个连接
     *
     * @return
     */
    @Override
    public Connection getConnection() {
        if (dataSource == null) {
            throw new RuntimeException("the dataSource can not be null");
        }
        Connection connection = connectionThreadLocal.get();
        if (connection == null) {
            try {
                connectionThreadLocal.set(dataSource.getConnection());
                connection = connectionThreadLocal.get();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
        return connection;
    }

}
