package com.rubin.spring.mybatis.handler;

import com.rubin.mybatis.sqlsession.SqlSessionFactory;
import com.rubin.spring.context.factory.AutowiredBeanFactory;
import com.rubin.spring.context.handler.JdkProxyHandler;
import lombok.AllArgsConstructor;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;

/**
 * mapper代理逻辑实现
 */
@AllArgsConstructor
public class MapperInvocationHandler implements InvocationHandler, JdkProxyHandler {

    private AutowiredBeanFactory autowiredBeanFactory;

    private Class<?> targetClass;

    @Override
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        SqlSessionFactory sqlSessionFactory = (SqlSessionFactory) autowiredBeanFactory.getBean(SqlSessionFactory.class);
        if (sqlSessionFactory == null) {
            throw new RuntimeException("the SqlSessionFactory must be defined");
        }
        Object mapper = sqlSessionFactory.openSession().getMapper(method.getDeclaringClass());
        Class[] paramTypeArr = new Class[args.length];
        for (int i = 0; i < args.length; i++) {
            paramTypeArr[i] = args[i].getClass();
        }
        Method mapperMethod = mapper.getClass().getMethod(method.getName(), paramTypeArr);
        return mapperMethod.invoke(mapper, args);
    }

    /**
     * 获取代理目标类类型
     *
     * @return
     */
    @Override
    public Class<?> getTargetClass() {
        return targetClass;
    }

}
