package com.rubin.spring.mybatis.processor;

import com.rubin.spring.context.factory.AutowiredBeanFactory;
import com.rubin.spring.context.processor.BeanFactoryPostProcessor;
import com.rubin.spring.context.utils.BeanUtil;
import com.rubin.spring.context.utils.CollectionUtil;
import com.rubin.spring.context.utils.ProxyUtil;
import com.rubin.spring.context.utils.ReflectUtil;
import com.rubin.spring.mybatis.annotation.Mapper;
import com.rubin.spring.mybatis.handler.MapperInvocationHandler;
import com.rubin.spring.mybatis.scanner.MapperScanner;

import java.util.Set;

/**
 * 自定义容器处理器
 * Created by rubin on 4/3/21.
 */
public class MybatisBeanFactoryProcessor implements BeanFactoryPostProcessor {

    /**
     * 自动驻入之前的钩子函数,此处扫描mapper接口，并代理注入容器
     *
     * @param autowiredBeanFactory
     */
    @Override
    public void postProcessBeanFactory(AutowiredBeanFactory autowiredBeanFactory) {
        MapperScanner mapperScanner = (MapperScanner) autowiredBeanFactory.getBean(MapperScanner.class);
        if (mapperScanner == null) {
            throw new RuntimeException("the MapperScanner must be defined");
        }
        Set<Class> mapperClassSet = ReflectUtil.getAllInterfaceWithAnnotation(mapperScanner.getMapperLocation(), Mapper.class);
        if (CollectionUtil.isEmpty(mapperClassSet)) {
            return;
        }
        mapperClassSet.stream().forEach(mapperClass -> injectMapperProxy2BeanFactory(mapperClass, autowiredBeanFactory));
    }

    /**
     * 将mapper生成代理类并注入到容器中
     *
     * @param mapperClass
     * @param autowiredBeanFactory
     */
    private void injectMapperProxy2BeanFactory(Class mapperClass, AutowiredBeanFactory autowiredBeanFactory) {
        String beanId = BeanUtil.getDefaultBeanName(mapperClass.getSimpleName());
        Object beanValue = ProxyUtil.getInterfaceJdkProxy(mapperClass, new MapperInvocationHandler(autowiredBeanFactory, mapperClass));
        autowiredBeanFactory.registerBean(beanId, beanValue);
    }

    /**
     * 执行顺序 升序排列执行
     *
     * @return
     */
    @Override
    public Integer getOrder() {
        return 0;
    }

}
